using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace CustomEventsAndDelegates
{
    public delegate void StatusChangedHandler(object sender, StatusChangedEventArgs e);

    public class StatusChangedEventArgs : EventArgs
    {
        // This will store our only parameter / event argument, which is the event message
        private string EventMsg;

        // We need to define this property in order to retrieve the message in the event handler, back in Form1.cs
        public string EventMessage
        {
            get
            {
                return EventMsg;
            }
        }

        // The constructor will set the message
        public StatusChangedEventArgs(string strEventMsg)
        {
            EventMsg = strEventMsg;
        }
    }

    class Monitor
    {
        // Will store the IP address passed to it
        IPAddress ipAddress;

        // The constructor sets the IP address to the one retrieved by the instantiating object
        public Monitor(IPAddress address)
        {
            ipAddress = address;
        }

        // Declare the event that we'll fire later
        public event StatusChangedHandler StatusChangedEvent;
        // The thread that will hold the connection listener
        private Thread thrListener;
        // The TCP object that listens for connections
        private TcpListener tlsServer;
        // The thread that will send information to the client
        private Thread thrSender;
        // Will tell the while loop to keep monitoring for connections
        bool ServRunning = false;

        public void StartMonitoring()
        {
            // Get the IP of the first network device, however this can prove unreliable on certain configurations
            IPAddress ipaLocal = ipAddress;
            if (tlsServer == null)
            {
                // Create the TCP listener object using the IP of the server and the specified port
                tlsServer = new TcpListener(ipaLocal, 1986);
            }
            // Start the TCP listener and listen for connections
            tlsServer.Start();

            // The while loop will check for true in this before checking for connections
            ServRunning = true;

            // Start the new tread that hosts the listener
            thrListener = new Thread(KeepListening);
            thrListener.Start();
        }

        private void KeepListening()
        {
            TcpClient tclServer;
            // While the server is running
            while (ServRunning == true)
            {
                // Accept a pending connection
                tclServer = tlsServer.AcceptTcpClient();
                // Start a new thread where our new client who just connected will be managed
                thrSender = new Thread(new ParameterizedThreadStart(AcceptClient));
                // The thread calls the AcceptClient() method
                thrSender.Start(tclServer);
            }
        }

        // Occures when a new client is accepted
        private void AcceptClient(object newClient)
        {
            // Set the argument/parameter to a message explaining what just happened
            StatusChangedEventArgs evArg = new StatusChangedEventArgs("A client was successfully accepted.");
            // Fire the event because a new client was accepted
            StatusChangedEvent(this, evArg);
        }
    }
}
